<?php

/**
 * This file is a part of Core - set of reusable tools and code.
 * All rights reserved.
 *
 * Developed by SourceModders.
 */

namespace SModders\Core\AddOn;


use SModders\Core\Install\Data\AbstractData;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;

abstract class AbstractSetup extends \XF\AddOn\AbstractSetup
{
    use StepRunnerInstallTrait;
    use StepRunnerUpgradeTrait;
    use StepRunnerUninstallTrait;
    
    protected function installTables()
    {
        $this->getInstallData()
            ->apply();
    }
    
    protected function dropTables()
    {
        $this->getInstallData()
            ->drop();
    }
    
    /**
     * @return AbstractData
     * @throws \Exception
     */
    protected function getInstallData()
    {
        /** @var \SModders\Core\Data\DbDriver $dbDriverResolver */
        $dbDriverResolver = $this->data('SModders\Core:DbDriver');
        $driverName = $dbDriverResolver->resolveAdapterName($this->db());
    
        if (empty($driverName))
        {
            throw new \Exception('Unknown database driver is used');
        }
    
        $rootAddOnNamespace = $this->getRootNamespace();
        $identifier = sprintf('%s:%s', $rootAddOnNamespace, $driverName);
        $dataClass = $this->stringAndExtendClass($identifier, '%s\Install\Data\%s');
        
        return new $dataClass($this->schemaManager());
    }
    
    /**
     * @return string
     */
    protected function getRootNamespace()
    {
        return $this->addOn->prepareAddOnIdForClass();
    }
    
    /**
     * @param $identifier
     * @return mixed
     * @throws \Exception
     */
    protected function data($identifier)
    {
        $class = $this->stringAndExtendClass($identifier, '%s\Data\%s');
        return new $class();
    }
    
    /**
     * @param $string
     * @param $formatter
     * @param null|string $defaultInfix
     * @param null|string $fakeBaseClass
     * @return string
     * @throws \Exception
     */
    protected function stringAndExtendClass($string, $formatter, $defaultInfix = null, $fakeBaseClass = null)
    {
        $class = \XF::stringToClass($string, $formatter, $defaultInfix);
        return $this->app->extendClass($class, $fakeBaseClass);
    }

    /**
     * @param string $uniqueId
     * @param string $jobClass
     * @param string|null $requiredAddOnId
     * @return int|null
     */
    protected function enqueueJob($uniqueId, $jobClass, $requiredAddOnId = null)
    {
        return $this->enqueueJobWithParameters($uniqueId, $jobClass, [], $requiredAddOnId);
    }

    /**
     * @param string $uniqueId
     * @param string $jobClass
     * @param array $params
     * @param string|null $requiredAddOnId
     * @return int|null
     */
    protected function enqueueJobWithParameters($uniqueId, $jobClass, array $params, $requiredAddOnId = null)
    {
        // Ensure required addon is installed.
        if ($requiredAddOnId != null)
        {
            $addOn = $this->app()->addOnManager()->getById($requiredAddOnId);
            if ($addOn == null || $addOn->isInstalled() == false)
            {
                return;
            }
        }

        // Ensure this is not full job class name.
        if (strpos($jobClass, ':'))
        {
            $jobClass = $this->getRootNamespace() . ':' . $jobClass;
        }

        return $this->app()->jobManager()->enqueueUnique(
            'smodders_' . $uniqueId,
            $jobClass,
            $params,
            false
        );
    }
}
