<?php

/**
 * This file is a part of Core - set of reusable tools and code.
 * All rights reserved.
 *
 * Developed by SourceModders.
 */

namespace SModders\Core;


class AddOnListener
{
    /**
     * Called when the post-rebuild code for an add-on has been run.
     * This event listener initiates opcode cache invalidate for all addOn files.
     *
     * @param \XF\AddOn\AddOn $addOn
     * @param \XF\Entity\AddOn $installedAddOn
     * @param array $json
     */
    public static function onAddOnPostRebuild(\XF\AddOn\AddOn $addOn, \XF\Entity\AddOn $installedAddOn, array $json)
    {
        self::invalidateAddOnOpcodeCache($addOn);
    }
    
    protected static function invalidateAddOnOpcodeCache(\XF\AddOn\AddOn $addOn)
    {
        $rootDirectory = \XF::getRootDirectory() . \XF::$DS;
        foreach (array_keys($addOn->getHashes()) as $filePath)
        {
            \XF\Util\Php::invalidateOpcodeCache($rootDirectory . $filePath);
        }
    }
}