<?php

/**
 * This file is a part of Core - set of reusable tools and code.
 * All rights reserved.
 *
 * Developed by SourceModders.
 */

namespace SModders\Core\Install\Data;


use XF\Db\SchemaManager;

abstract class AbstractData
{
    /** @var SchemaManager */
    protected $sm;
    
    public function __construct(SchemaManager $sm)
    {
        $this->sm = $sm;
    }

    public function apply($tableName = null)
    {
        $tablePrefix = $this->getTablePrefix();
        $tables = $this->getTables();
        if ($tableName !== null)
        {
            $tables = [$tableName => $tables[$tableName]];
        }

        foreach ($tables as $tableName => $closure)
        {
             $this->sm->createTable($tablePrefix . $tableName, $closure);
        }
    }
    
    public function drop()
    {
        $tablePrefix = $this->getTablePrefix();
        foreach (array_keys($this->getTables()) as $tableName)
        {
            $this->sm->dropTable($tablePrefix . $tableName);
        }
    }
    
    /**
     * @return array
     */
    protected abstract function getTables();
    
    /**
     * @return string
     */
    protected function getTablePrefix($withUnderscore = true)
    {
        return 'xf_' . $this->_getTablePrefix() . ($withUnderscore ? '_' : '');
    }
    
    /**
     * @return string
     */
    protected abstract function _getTablePrefix();
}